@@ -70,67 +70,72 @@ static int set_aspect(int x, int y)
      return (ox != xaspect || oy != yaspect);
 }
 
-__inline
-static void max_aspect_window(Display *display, Window window, int w, int h)
-{
-     int tmpw, tmph;
-     XWindowAttributes ca;
-
-     XGetWindowAttributes(display, window, &ca);
-
-     /* Select the side (x or y) to use to rescale */
-     tmpw = w / xaspect;
-     tmph = h / yaspect;
-     if (tmpw < tmph) tmph = tmpw;
-
-     /* Compute the size of the new window */
-     tmpw = tmph * xaspect;
-     tmph = tmph * yaspect;
-     
-     D("New size: %dx%d+%d+%d (of %dx%d)\n", tmpw, tmph,
-       (w - tmpw)/2,
-       (h - tmph)/2,
-       w, h);
-
-     XMoveResizeWindow(display, window,
-		       (w - tmpw)/2,
-		       (h - tmph)/2,
-		       tmpw, 
-		       tmph);
-}
-
-__inline
-static void fill_window(Display *display, Window window, int w, int h)
-{
-     D("New size: %dx%d+%d+%d (of %dx%d)\n", w, h, 0, 0, w, h);
-     XMoveResizeWindow(display, window, 0, 0, w, h);
-}
-
 static void adjust_window_size(void)
 {
+     static int old_x = 0;
+     static int old_y = 0;
+     static int old_w = 0;
+     static int old_h = 0;
+
+     int x = 0;
+     int y = 0;
      int w = windata.width;
      int h = windata.height;
-     Window window = victim;
+
+     int tmpw, tmph;
+
+     if (!victim) return;
+     if (!WINDOW) return;
+     if (old_parent != WINDOW) return;
 
      if (flags & H_FILL)
      {
-	  D("Resizing window %x with FILL\n", window);
-	  fill_window(display, window, w, h);
+	  D("Resizing window %x with FILL\n", victim);
      }
      else if (flags & H_MAXASPECT)
      {
-	  D("Resizing window %x with MAXASPECT\n", window);
-	  max_aspect_window(display, window, w, h);
-     } else {
-	  D("Not resizing window\n");
+	  if (xaspect && yaspect)
+	  {
+	       D("Resizing window %x with MAXASPECT\n", victim);
+	       tmph = h / yaspect;
+	       tmpw = w / xaspect;
+	       if (tmpw < tmph) tmph = tmpw;
+	       tmpw = tmph * xaspect;
+	       tmph = tmph * yaspect;
+	       
+	       x = (w - tmpw) / 2;
+	       y = (h - tmph) / 2;
+	       
+	       w = tmpw;
+	       h = tmph;
+	  }
+	  else
+	  {
+	       D("Not resizing window\n");
+	       return;
+	  }
      }
+  
+     D("New size: %dx%d+%d+%d\n", w, h, x, y);
+
+     /* Hack to get X to actually resize the window when mozplugger
+      * resizes it two times with the same dimensions. */
+     if (old_x == x && old_y == y && old_h == h && old_w == w)
+	  XMoveResizeWindow(display, victim, x, y, w, h+1);
+     
+     XMoveResizeWindow(display, victim, x, y, w, h);
+     
+     old_x = x;
+     old_y = y;
+     old_h = h;
+     old_w = w;
 }
 
 /*****************************************************************************
  * Window reparenting
  *****************************************************************************/
 
-static int place_window(void)
+static int reparent_window(void)
 {
      XWMHints *leader_change;
 
@@ -154,7 +159,7 @@ static int place_window(void)
      D("Reparenting window %x into %x\n", victim, WINDOW);
      XReparentWindow(display, victim, WINDOW, 0, 0);
 
-     D("place_window() done\n");
+     D("reparent_window() done\n");
 
      return 1;
 }
@@ -281,12 +286,12 @@ int initial_win_cb(Window w)
 /* Returns 1 if and only if w is a new window. */
 int win_cb(Window w)
 {
-  int i;
-  for (i=0; i < windows_found; i++) {
-       if (ignored[i] == w)
-	    return 0;
-  }
-  return 1;
+     int i;
+     for (i=0; i < windows_found; i++) {
+	  if (ignored[i] == w)
+	       return 0;
+     }
+     return 1;
 }
 
 void save_initial_windows()
@@ -359,7 +364,7 @@ static void find_victim(Window window)
 	       }
 
 	       old_parent = 0;
-	       place_window();
+	       reparent_window();
 	  }
      }
 }
@@ -391,12 +396,12 @@ static void check_x_events(void)
 		    break;
 
 	       case MapNotify:
-		    D("***MapNotify: %x\n",ev.xmap.window);
+		    D("***MapNotify: %x\n", ev.xmap.window);
 		    find_victim(ev.xmap.window);
 		    break;
 
 	       case ConfigureNotify:
-		    D("***ConfigureNotify: %x\n",ev.xconfigure.window);
+		    D("***ConfigureNotify: %x\n", ev.xconfigure.window);
 		    find_victim(ev.xconfigure.window);
 		    break;
 	       }
@@ -423,7 +428,7 @@ static void check_x_events(void)
 			 XMapWindow(display, victim);
 		    } else {
 			 D("REPARENT NOTIFY to some other window!\n");
-			 place_window();
+			 reparent_window();
 		    }
 		    break;
 	       }
