@@ -20,14 +20,13 @@
 #include "mozplugger.h"
 
 #define THIS (*(data_t **) &(instance->pdata))
-#define FEND(X,Y) (strchr(X,Y)?strchr(X,Y):(X)+strlen(X))
 
 /*****************************************************************************
  * Type declarations
  *****************************************************************************/
 typedef struct argument
 {
-     char *var;
+     char *name;
      char *value;
 } argument_t;
 
@@ -53,24 +52,23 @@ typedef struct data
 
 typedef struct mimetype
 {
-     struct mimetype *next;
-     char *line;
+     char type[MAX_TYPE_SIZE];
 } mimetype_t;
 
 typedef struct command
 {
-     struct command *next;
      int flags;
-     char *cmd;
-     char *winname;
+     char cmd[MAX_CMD_SIZE];
+     char winname[MAX_WINNAME_SIZE];
 } command_t;
 
 typedef struct handle
 {
-     struct handle *next;
-     struct mimetype *types;
-     struct command *commands;
-} handle_t;
+     int num_types;
+     int num_cmds;
+     mimetype_t types[MAX_NUM_TYPES_PER_HANDLER];
+     command_t cmds[MAX_NUM_CMDS_PER_HANDLER];
+} handler_t;
 
 typedef struct
 {
@@ -84,9 +82,8 @@ typedef struct
 static char *config_fname;
 static char *helper_fname;
 static char *controller_fname;
-static struct handle *first_handle;
-static struct handle **prev_handlep;
-static struct handle **handlep;
+static int num_handlers;
+static handler_t handlers[MAX_NUM_HANDLERS];
 
 /*****************************************************************************
  * Wrapper for fork() which handles some magic needed to prevent netscape
@@ -96,6 +93,7 @@ static int my_fork(NPP instance)
 {
      int pid;
      sigset_t set,oset;
+     int signum;
 
      /* Mask all the signals to avoid being interrupted by a signal */
      sigfillset(&set);
@@ -104,11 +102,12 @@ static int my_fork(NPP instance)
      D(">>>>>>>>Forking<<<<<<<<,\n");
 
      if ((pid = fork()) == -1)
+     {
 	  return -1;
+     }
 
      if (!pid)
      {
-	  int signum;
 	  alarm(0);
 
 	  if (!(THIS->flags & H_DAEMON))
@@ -117,12 +116,15 @@ static int my_fork(NPP instance)
 	  for (signum=0; signum < NSIG; signum++)
 	       signal(signum, SIG_DFL);
 
+	  THIS->display = NULL;
+
 #ifdef DEBUG
 	  close_debug();
 	  D("Child re-opened ndebug\n");
 #endif
-	  THIS->display=0;
-     } else {
+     }
+     else
+     {
 	  D("Child running with pid=%d\n", pid);
      }
 
@@ -135,11 +137,10 @@ static int my_fork(NPP instance)
 /*****************************************************************************
  * Wrapper for putenv().
  *****************************************************************************/
-#define ENV_BUFFER 16348
 static void my_putenv(char *buffer, int *offset, const char *var, const char *value)
 {
      int l = strlen(var) + strlen(value) + 2;
-     if (!(*offset + l < ENV_BUFFER))
+     if (!(*offset + l < ENV_BUFFER_SIZE))
      {
 	  D("Buffer overflow in putenv(%s=%s)\n", var, value);
 	  return;
@@ -155,8 +156,8 @@ static void my_putenv(char *buffer, int *offset, const char *var, const char *va
  *****************************************************************************/
 static void run(NPP instance, const char *file)
 {
-     char buffer[ENV_BUFFER];
-     char foo[64];
+     char buffer[ENV_BUFFER_SIZE];
+     char foo[SMALL_BUFFER_SIZE];
      int offset = 0;
      int i;
 
@@ -215,7 +216,7 @@ static void run(NPP instance, const char *file)
 
      for (i = 0; i < THIS->num_arguments; i++)
 	  if (THIS->args[i].value)
-	       my_putenv(buffer, &offset, THIS->args[i].var, THIS->args[i].value);
+	       my_putenv(buffer, &offset, THIS->args[i].name, THIS->args[i].value);
 
      execlp(helper_fname, helper_fname, buffer, THIS->command, NULL);
 
@@ -289,14 +290,16 @@ static int find(char *file)
  *****************************************************************************/
 static void filter_previous_handler()
 {
-     if (prev_handlep && (*prev_handlep) && !(*prev_handlep)->commands)
+     handler_t *h;
+     if (num_handlers >= 1)
      {
-	  handlep = prev_handlep;
-	  free((*prev_handlep)->types->line);
-	  free((*prev_handlep)->types);
-	  free((*prev_handlep));
-	  *prev_handlep = NULL;
-	  prev_handlep = NULL;
+	  h = &handlers[num_handlers-1];
+	  if (h->num_cmds == 0)
+	  {
+	       D("Empty handler: '%s'.\n", h->types[0].type);
+	       h->num_types = 0;
+	       num_handlers--;
+	  }
      }
 }
 
@@ -306,34 +309,39 @@ static void filter_previous_handler()
 static char *get_winname(char *x, char *flag, command_t *c)
 {
      char *end;
+     int len;
  
      x += strlen(flag);
      while (isspace(*x)) x++;
      if (*x != '(')
      {
-	  fprintf(stderr,"Expected '(' after '%s'\n", flag);
+	  fprintf(stderr, "MozPlugger: Warning: Expected '(' after '%s'\n", flag);
 	  return NULL;
      }
      x++;
      end = strchr(x,')');
      if (end)
      {
-	  if (!(c->winname = malloc(end - x + 1)))
-	       return NULL;
-	  memcpy(c->winname, x, end-x);
-	  c->winname[end-x]=0;
+	  len = end - x;
+	  if (!(len < sizeof(c->winname)))
+	  {
+	       fprintf(stderr, "MozPlugger: Error: Window name too long (%s)\n", x);
+	       exit(1);
+	  }
+	  memcpy(c->winname, x, len);
+	  c->winname[len] = 0;
 	  x = end+1;
      }
      return x;
 }
 
 /*****************************************************************************
- * Match a word to a single-word flag.
+ * Match two words.
  *****************************************************************************/
 __inline
-static int match(char *line, char *kw)
+static int match_word(char *line, char *kw)
 {
-     return !strncasecmp(line,kw,strlen(kw)) && !isalnum(line[strlen(kw)]);
+     return !strncasecmp(line, kw, strlen(kw)) && !isalnum(line[strlen(kw)]);
 }
 
 /*****************************************************************************
@@ -360,22 +368,22 @@ static int parse_flags(char **x, command_t *commandp)
 
      for (f = flags; f->name; f++)
      {
-	  if (match(*x, f->name))
+	  if (match_word(*x, f->name))
 	  {
 	       *x += strlen(f->name);
 	       commandp->flags |= f->value;
 	       return 1;
 	  }
-	  if (match(*x, "swallow"))
+	  if (match_word(*x, "swallow"))
 	  {
 	       commandp->flags |= H_SWALLOW;
 	       if ((*x = get_winname(*x, "swallow", commandp)))
 		    return 1;
 	  }
-	  if (match(*x, "controls"))
+	  if (match_word(*x, "controls"))
 	  {
 	       commandp->flags |= H_CONTROLS | H_FILL | H_SWALLOW | H_HIDDEN;
-	       commandp->winname = strdup("mozplugger-controller");
+	       strcpy(commandp->winname, "mozplugger-controller");
 	       *x += 8;
 	       return 1;
 	  }
@@ -388,66 +396,99 @@ static int parse_flags(char **x, command_t *commandp)
  *****************************************************************************/
 static void read_config(FILE *f)
 {
-     command_t **commandp = NULL;
-     mimetype_t **typep = NULL;
-     char buffer[16384];
-     char file[128];
+     handler_t *handler = NULL;
+     command_t *cmd = NULL;
+     mimetype_t *type = NULL;
+     char buffer[LARGE_BUFFER_SIZE];
+     char file[SMALL_BUFFER_SIZE];
      int have_commands = 1;
 
      D("read_config\n");
 
-     handlep = &first_handle;
-
-     while (!feof(f))
+     while (fgets(buffer, sizeof(buffer), f))
      {
-	  fgets(buffer, sizeof(buffer), f);
 	  D("::: %s", buffer);
 
-	  if (buffer[0]=='#' || !buffer[0] || buffer[0]=='\n')
+	  if (buffer[0] == '#' || buffer[0] == '\n' || buffer[0] == '\0')
 	       continue;
 
 	  if (buffer[strlen(buffer)-1] == '\n')
 	       buffer[strlen(buffer)-1] = 0;
 
-	  if (isspace(buffer[0]))
+	  if (!isspace(buffer[0]))
 	  {
+	       /* Mime type */
+
+	       if (have_commands)
+	       {
+		    D("-------------------------------------------\n");
+		    D("New handler\n");
+		    D("-------------------------------------------\n");
+
+		    filter_previous_handler();
+
+		    handler = &handlers[num_handlers++];
+
+		    if (!(num_handlers < MAX_NUM_HANDLERS))
+		    {
+			 fprintf(stderr, "MozPlugger: Error: Too many handlers (%d)\n", num_handlers);
+			 exit(1);
+		    }
+
+		    have_commands = 0;
+	       }
+
+	       D("New mime type\n");
+
+	       type = &handler->types[handler->num_types++];
+
+	       if (!(handler->num_types < MAX_NUM_TYPES_PER_HANDLER))
+	       {
+		    fprintf(stderr, "MozPlugger: Error: Too many types (%d) for handler %d (%s)\n",
+			    handler->num_types, num_handlers, handler->types[0].type);
+		    exit(1);
+	       }
+
+	       if (!(strlen(buffer) < sizeof(type->type)))
+	       {
+		    fprintf(stderr, "MozPlugger: Error: Mime type too long (%s)\n", buffer);
+		    exit(1);
+	       }
+
+	       strncpy(type->type, buffer, sizeof(type->type));
+	  }
+	  else
+	  {
+	       /* Command */
+
 	       char *x = buffer + 1;
 	       while (isspace(*x)) x++;
 	       if (!*x)
 	       {
-		    D("Empty line.\n");
+		    D("Empty command.\n");
+		    have_commands++;
 		    continue;
 	       }
 
-	       /* Command */
 	       D("New command\n");
 
-	       if (!(*commandp = (command_t *)malloc(sizeof(command_t))))
-		    return;
-	       (*commandp)->flags = 0;
-	       (*commandp)->cmd = NULL;
-	       (*commandp)->winname = NULL;
-	       (*commandp)->next = NULL;
-
 	       have_commands++;
 
+	       cmd = &handler->cmds[handler->num_cmds];
+	       memset(cmd, 0, sizeof(command_t));
+
 	       D("Parsing %s\n", x);
 
 	       while (*x != ':' && *x)
 	       {
-		    switch(*x) {
-		    case ',':
-		    case ' ':
-		    case '\t':
+		    if (*x == ',' || *x == ' ' || *x == '\t')
+		    {
 			 x++;
-			 break;
-		    default:
-			 if (!parse_flags(&x, *commandp))
-			 {
-			      D("Unknown directive: %s\n", x);
-			      fprintf(stderr,"Unknown directive: %s\n", x);
-			      x += strlen(x);
-			 }
+		    }
+		    else if (!parse_flags(&x, cmd))
+		    {
+			 fprintf(stderr, "MozPlugger: Warning: Unknown directive: %s\n", x);
+			 x += strlen(x);
 		    }
 	       }
 	       if (*x == ':')
@@ -455,73 +496,40 @@ static void read_config(FILE *f)
 		    x++;
 		    while (isspace(*x)) x++;
 
-		    if (sscanf(x, "if %s", file) != 1
-			&& sscanf(x, "%s", file) != 1)
+		    if (sscanf(x, "if %"SMALL_BUFFER_SIZE_STR"s", file) != 1
+			&& sscanf(x, "%"SMALL_BUFFER_SIZE_STR"s", file) != 1)
 			 continue;
 
 		    if (!find(file))
-		    {
-			 free((*commandp)->winname);
-			 free(*commandp);
-			 *commandp = NULL;
 			 continue;
+
+		    if (!(strlen(x) < sizeof(cmd->cmd)))
+		    {
+			 fprintf(stderr, "MozPlugger: Error: Command too long (%s)\n", x);
+			 exit(1);
 		    }
 
-		    (*commandp)->cmd = strdup(x);
-	       } else {
+		    strncpy(cmd->cmd, x, sizeof(cmd->cmd));
+	       }
+	       else
+	       {
 		    D("No column? (%s)\n", x);
 	       }
 
-	       if (!(*commandp)->cmd)
+	       handler->num_cmds++;
+
+	       if (!(handler->num_cmds < MAX_NUM_CMDS_PER_HANDLER))
 	       {
-		    free((*commandp)->winname);
-		    free(*commandp);
-		    *commandp = NULL;
-		    continue;
+		    fprintf(stderr, "MozPlugger: Error: Too many commands (%d) for handler %d (%s)\n",
+			    handler->num_cmds, num_handlers, handler->types[0].type);
+		    exit(1);
 	       }
-
-	       commandp=&((*commandp)->next);
-	  } else {
-	       /* Mime type */
-
-	       if (have_commands)
-	       {
-		    D("New handle\n");
-		    if (commandp) D("Commandp=%p\n", *commandp);
-
-		    filter_previous_handler();
-
-		    if (!(*handlep = (handle_t *)malloc(sizeof(handle_t))))
-			 return;
-
-		    (*handlep)->commands = NULL;
-		    (*handlep)->types = NULL;
-		    (*handlep)->next = NULL;
-		    commandp = &((*handlep)->commands);
-		    typep = &((*handlep)->types);
-		    prev_handlep = handlep;
-		    handlep = &((*handlep)->next);
-		    have_commands = 0;
-	       }
-
-	       D("New mimetype\n");
-	       if (!(*typep = (mimetype_t *)malloc(sizeof(mimetype_t))))
-		    return;
-
-	       (*typep)->next = NULL;
-	       (*typep)->line = strdup(buffer);
-
-	       if (!(*typep)->line)
-	       {
-		    free(*typep);
-		    *typep = NULL;
-		    return;
-	       }
-	       typep = &((*typep)->next);
 	  }
      }
 
      filter_previous_handler();
+
+     D("Num handlers: %d\n", num_handlers);
 }
 
 /*****************************************************************************
@@ -529,7 +537,7 @@ static void read_config(FILE *f)
  *****************************************************************************/
 static int find_helper_file(char *basename, int (*cb)(char *,void *data), void *data)
 {
-     static char fname[8192];
+     static char fname[LARGE_BUFFER_SIZE];
      char *tmp;
 
      D("find_helper_file '%s'\n", basename);
@@ -538,8 +546,10 @@ static int find_helper_file(char *basename, int (*cb)(char *,void *data), void *
      {
 	  snprintf(fname, sizeof(fname), "%s/.netscape/%s", tmp, basename);
 	  if (cb(fname,data)) return 1;
+
 	  snprintf(fname, sizeof(fname), "%s/.mozilla/%s", tmp, basename);
 	  if (cb(fname,data)) return 1;
+
 	  snprintf(fname, sizeof(fname), "%s/.opera/%s", tmp, basename);
 	  if (cb(fname,data)) return 1;
      }
@@ -558,12 +568,16 @@ static int find_helper_file(char *basename, int (*cb)(char *,void *data), void *
 
      snprintf(fname, sizeof(fname), "/etc/%s", basename);
      if (cb(fname, data)) return 1;
+
      snprintf(fname, sizeof(fname), "/usr/etc/%s", basename);
      if (cb(fname, data)) return 1;
+
      snprintf(fname, sizeof(fname), "/usr/local/mozilla/%s", basename);
      if (cb(fname, data)) return 1;
+
      snprintf(fname, sizeof(fname), "/usr/local/netscape/%s", basename);
      if (cb(fname, data)) return 1;
+
      if (cb(basename, data)) return 1;
   
      return 0;
@@ -574,7 +588,7 @@ static int read_config_cb(char *fname, void *data)
      int m4out[2];
      int pid;
      int fd;
-     FILE *f;
+     FILE *fp;
 
      D("READ_CONFIG(%s)\n", fname);
 
@@ -592,27 +606,32 @@ static int read_config_cb(char *fname, void *data)
 
      if (!pid)
      {
-	  dup2(fd, 0);
-	  dup2(m4out[1], 1);
+	  close(m4out[0]);
 
-	  close(fd);
+	  dup2(m4out[1], 1);
 	  close(m4out[1]);
 
+	  dup2(fd, 0);
+	  close(fd);
+
 	  execlp("m4", "m4", NULL);
-	  fprintf(stderr, "MozPlugger: Failed to execute m4.\n");
+	  fprintf(stderr, "MozPlugger: Error: Failed to execute m4.\n");
 	  exit(1);
      }
+     else
+     {
+	  close(m4out[1]);
+	  close(fd);
+	  
+	  fp = fdopen(m4out[0], "r");
+	  if (!fp) return 0;
+	  read_config(fp);
+	  fclose(fp);
+	  
+	  waitpid(pid, 0, 0);
+	  config_fname = strdup(fname);
+     }
 
-     close(fd);
-     close(m4out[1]);
-
-     f = fdopen(m4out[0], "r");
-     if (!f) return 0;
-     read_config(f);
-     fclose(f);
-     
-     waitpid(pid, 0, 0);
-     config_fname = strdup(fname);
      return 1;
 }
 
@@ -627,7 +646,7 @@ static int find_plugger_helper_cb(char *fname, void *data)
 static int find_plugger_controller_cb(char *fname, void *data)
 {
      struct stat buf;
-     if(stat(fname, &buf)) return 0;
+     if (stat(fname, &buf)) return 0;
      controller_fname = strdup(fname);
      return 1;
 }
@@ -637,13 +656,13 @@ static int find_plugger_controller_cb(char *fname, void *data)
  *****************************************************************************/
 static void do_read_config(void)
 {
-     if (first_handle) return;
+     if (num_handlers > 0) return;
 
      D("do_read_config\n");
   
      if (!find_helper_file("mozpluggerrc", read_config_cb, 0))
      {
-	  fprintf(stderr, "MozPlugger: Unable to find the mozpluggerrc file.\n");
+	  fprintf(stderr, "MozPlugger: Warning: Unable to find the mozpluggerrc file.\n");
 	  return;
      }
 
@@ -652,7 +671,7 @@ static void do_read_config(void)
 	  if (find("mozplugger-helper")) {
 	       helper_fname = "mozplugger-helper";
 	  } else { 
-	       fprintf(stderr, "MozPlugger: Unable to find mozplugger-helper.\n");
+	       fprintf(stderr, "MozPlugger: Warning: Unable to find mozplugger-helper.\n");
 	       return;
 	  }
      }
@@ -662,7 +681,7 @@ static void do_read_config(void)
 	  if (find("mozplugger-controller")) {
 	       controller_fname = "mozplugger-controller";
 	  } else {
-	       fprintf(stderr, "MozPlugger: Unable to find mozplugger-controller.\n");
+	       fprintf(stderr, "MozPlugger: Warning: Unable to find mozplugger-controller.\n");
 	       return;
 	  }
      }
@@ -682,7 +701,7 @@ static int safeURL(char* url)
      if (url[0] == '/')
 	  return 0;
 
-     for(i = 0; i < len; i++)
+     for (i = 0; i < len; i++)
      {
 	  if (url[i] == '`' || url[i] == ';')
 	  {
@@ -696,68 +715,105 @@ static int safeURL(char* url)
 /*****************************************************************************
  * Go through the commands in the config file and find one that fits our needs.
  *****************************************************************************/
-static int find_command(NPP instance, int streaming)
+__inline
+static int match_command(NPP instance, int streaming, command_t *c)
 {
-     handle_t *h;
-     mimetype_t *m;
-     command_t *c;
      char embedded = (THIS->mode == NP_EMBED);
 
-     D("find_command\n");
+     D("Checking command: %s\n", c->cmd);
 
-     do_read_config();
-
-     D("find_command...\n");
-
-     for (h = first_handle; h; h = h->next)
+     if (embedded && (c->flags & H_NOEMBED))
      {
-	  D("commands for this handle at (%p)\n",h->commands);
-	  for (m = h->types; m; m = m->next)
+	  D("Flag mismatch: embed\n");
+	  return 0;
+     }
+     if ((!embedded) && (c->flags & H_EMBED))
+     {
+	  D("Flag mismatch: noembed\n");
+	  return 0;
+     }
+     if ((c->flags & H_LOOP) && (THIS->repeats != MAXINT))
+     {
+	  D("Flag mismatch: loop\n");
+	  return 0;
+     }
+     if ((!!streaming) != (!!(c->flags & H_STREAM)))
+     {
+	  D("Flag mismatch: stream\n");
+	  return 0;
+     }
+
+     D("Match found!\n");
+     return 1;
+}
+
+__inline
+static int match_mime_type(NPP instance, mimetype_t *m)
+{
+     char mimetype[SMALL_BUFFER_SIZE];
+     sscanf(m->type, "%"SMALL_BUFFER_SIZE_STR"[^:]", mimetype);
+     sscanf(mimetype, "%s", mimetype);
+
+     D("Checking '%s' ?= '%s'\n", mimetype, THIS->mimetype);
+     if (strcasecmp(mimetype, THIS->mimetype))
+     {
+	  D("Not same.\n");
+	  return 0;
+     }
+
+     D("Same.\n");
+     return 1;
+}
+
+__inline
+static int match_handler(handler_t *h, NPP instance, int streaming)
+{
+     mimetype_t *m;
+     command_t *c;
+     int mid, cid;
+
+     D("-------------------------------------------\n");
+     D("Commands for this handle at (%p):\n", h->cmds);
+
+     for (mid = 0; mid < h->num_types; mid++)
+     {
+	  m = &h->types[mid];
+	  if (match_mime_type(instance, m))
 	  {
-	       char *tmp1 = FEND(m->line,':');
-	       char tmp2;
-	       int tmp3;
-	       D("Checking '%s'\n", m->line);
-
-	       while (tmp1>m->line && isspace(tmp1[-1]))
-		    tmp1--;
-
-	       tmp2 = *tmp1;
-	       *tmp1 = 0;
-	       D("Checking '%s' ?= '%s'\n", m->line, THIS->mimetype);
-	       tmp3 = strcasecmp(THIS->mimetype, m->line);
-	       *tmp1 = tmp2;
-	       if (!tmp3)
+	       for (cid = 0; cid < h->num_cmds; cid++)
 	       {
-		    D("Match found!\n");
-		    break;
-	       } else {
-		    D("Not same.\n");
-	       }
-	  }
-
-	  if (m)
-	  {
-	       for (c = h->commands; c; c = c->next)
-	       {
-		    D("Checking command: %s\n", c->cmd);
-		    if (embedded && (c->flags & H_NOEMBED))
-			 continue;
-		    if ((!embedded) && (c->flags & H_EMBED))
-			 continue;
-		    if ((c->flags & H_LOOP) && (THIS->repeats != MAXINT))
-			 continue;
-		    if ((!!streaming) != (!!(c->flags & H_STREAM)))
-			 continue;
-		    D("Match found!\n");
-		    THIS->flags = c->flags;
-		    THIS->command = c->cmd;
-		    THIS->winname = c->winname;
-		    return 1;
+		    c = &h->cmds[cid];
+		    if (match_command(instance, streaming, c))
+		    {
+			 THIS->flags = c->flags;
+			 THIS->command = c->cmd;
+			 THIS->winname = c->winname;
+			 return 1;
+		    }
 	       }
 	  }
      }
-     D("No match found\n");
+     return 0;
+}
+
+static int find_command(NPP instance, int streaming)
+{
+     int hid;
+
+     D("find_command...\n");
+
+     do_read_config();
+
+     for (hid = 0; hid < num_handlers; hid++)
+     {
+	  if (match_handler(&handlers[hid], instance, streaming))
+	  {
+	       D("Command found.\n");
+	       return 1;
+	  }
+     }
+
+     D("No command found.\n");
      return 0;
 }
 
@@ -768,20 +824,29 @@ static int find_command(NPP instance, int streaming)
 char *NPP_GetMIMEDescription(void)
 {
      char *x,*y;
-     handle_t *h;
+     handler_t *h;
      mimetype_t *m;
      int size_required;
+     int hid, mid;
 
      D("GetMIMEDescription\n");
 
      do_read_config();
 
      size_required = 0;
-     for (h = first_handle; h; h = h->next)
-	  for (m = h->types; m; m = m->next)
-	       size_required += strlen(m->line)+1;
+
+     for (hid = 0; hid < num_handlers; hid++)
+     {
+	  h = &handlers[hid];
+	  for (mid = 0; mid < h->num_types; mid++)
+	  {
+	       m = &h->types[mid];
+	       size_required += strlen(m->type)+1;
+	  }
+     }
 
      D("Size required=%d\n", size_required);
+
      if (!(x = (char *)malloc(size_required+1)))
 	  return 0;
 
@@ -789,13 +854,14 @@ char *NPP_GetMIMEDescription(void)
 
      y = x;
 
-     for (h = first_handle; h; h = h->next)
+     for (hid = 0; hid < num_handlers; hid++)
      {
-	  D("Foo: %p\n", h->commands);
-	  for (m = h->types; m; m = m->next)
+	  h = &handlers[hid];
+	  for (mid = 0; mid < h->num_types; mid++)
 	  {
-	       memcpy(y,m->line,strlen(m->line));
-	       y+=strlen(m->line);
+	       m = &h->types[mid];
+	       memcpy(y,m->type,strlen(m->type));
+	       y+=strlen(m->type);
 	       *(y++)=';';
 	  }
      }
@@ -906,7 +972,7 @@ NPError NPP_New(NPMIMEType pluginType,
 
      THIS = NPN_MemAlloc(sizeof(data_t));
      if (!THIS) return NPERR_OUT_OF_MEMORY_ERROR;
-     memset((char *)THIS, 0, sizeof(*THIS));
+     memset((void *)THIS, 0, sizeof(data_t));
 
      THIS->windata.window = 0;
      THIS->display = NULL;
@@ -940,9 +1006,9 @@ NPError NPP_New(NPMIMEType pluginType,
 	  }
 	  /* copy the flag to put it into the environement later */
 	  D("VAR_%s=%s\n", argn[e], argv[e]);
-	  if (!(THIS->args[e].var = (char *)malloc(strlen(argn[e]) + 5)))
+	  if (!(THIS->args[e].name = (char *)malloc(strlen(argn[e]) + 5)))
 	       return NPERR_OUT_OF_MEMORY_ERROR;
-	  sprintf(THIS->args[e].var, "VAR_%s", argn[e]);
+	  sprintf(THIS->args[e].name, "VAR_%s", argn[e]);
 	  THIS->args[e].value = argv[e] ? strdup(argv[e]) : NULL;
      }
 
@@ -998,7 +1064,7 @@ NPError NPP_Destroy(NPP instance, NPSavedData** save)
 
 	  for (e = 0; e < THIS->num_arguments; e++)
 	  {
-	       free((char *)THIS->args[e].var);
+	       free((char *)THIS->args[e].name);
 	       free((char *)THIS->args[e].value);
 	  }
 
@@ -1053,7 +1119,9 @@ static void new_child(NPP instance, const char* fname)
 	  run(instance, fname);
 
 	  _exit(EX_UNAVAILABLE);
-     } else {
+     }
+     else
+     {
 	  THIS->fd = pipe[0];
 	  close(pipe[1]);
      }
@@ -1095,7 +1163,8 @@ NPError NPP_NewStream(NPP instance,
 
      D("Mime type %s\n", type);
      D("Url is %s (seekable=%d)\n", stream->url, seekable);
-     
+
+     /* A command may be removed after mozplugger has been loaded. */
      if (!find_command(instance, 1) && !find_command(instance, 0))
      {
 	  NPN_Status(instance, "MozPlugger: No appropriate application found.");
@@ -1164,9 +1233,11 @@ NPError NPP_SetWindow(NPP instance, NPWindow* window)
      {
 	  D("Writing WIN to fd %d\n", THIS->fd);
 	  write(THIS->fd, (char *)window, sizeof(*window));
-	  usleep(4000);
      }
 
+     /* In case Mozilla would call NPP_SetWindow() in a loop. */
+     usleep(4000);
+
      return NPERR_NO_ERROR;
 }
 
@@ -1215,7 +1286,8 @@ int32 NPP_WriteReady(NPP instance, NPStream *stream)
 {
      D("WriteReady\n");
 
-     if (!instance) {
+     if (!instance)
+     {
 	  D("No instance\n");
 	  return 0;
      }
