@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; -*- */
 /*
- *  npapi.h $Revision: 1.1 $
+ *  npapi.h $Revision: 1.2 $
  *  Netscape client plug-in API spec
  */
 
@@ -36,7 +36,7 @@
 typedef unsigned short uint16;
 #endif
 #ifndef _UINT32
-#if defined(__alpha) || defined(__amd64__)
+#if defined(__alpha) || defined(__amd64__) || defined(__ia64)
 typedef unsigned int uint32;
 #else /* __alpha */
 typedef unsigned long uint32;
@@ -46,7 +46,7 @@ typedef unsigned long uint32;
 typedef short int16;
 #endif
 #ifndef _INT32
-#if defined(__alpha) || defined(__amd64__)
+#if defined(__alpha) || defined(__amd64__) || defined(__ia64)
 typedef int int32;
 #else /* __alpha */
 typedef long int32;
